
//picorv32_demo
module picorv32_demo
(
  wbuart_tx,
  wbuart_rx,
  gpio_io,
  io_spi_clk,
  io_spi_csn,
  io_spi_mosi,
  io_spi_miso,
  irq_in,
  jtag_TDI,
  jtag_TDO,
  jtag_TCK,
  jtag_TMS,
  clk_in,
  resetn_in
);

//WB UART
output wbuart_tx;
input wbuart_rx;

//WB GPIO
inout [3:0] gpio_io;

//ADV SPI-Flash
inout io_spi_clk;
inout io_spi_csn;
inout io_spi_mosi;
inout io_spi_miso;

//External interrupts
input [1:0] irq_in; //irq[21],irq[20]

//JTAG
input jtag_TDI;
output jtag_TDO;
input jtag_TCK;
input jtag_TMS;

input clk_in;       //System clock
input resetn_in;    //System reset


//Open Wishbone bus
wire slv_ext_stb_o;
wire slv_ext_we_o;
wire slv_ext_cyc_o;
wire slv_ext_ack_i;
wire [31:0] slv_ext_adr_o;
wire [31:0] slv_ext_wdata_o;
wire [31:0] slv_ext_rdata_i;
wire [3:0] slv_ext_sel_o;

//Open AHB bus
wire [31:0] hrdata;
wire [1:0] hresp;
wire hready;
wire [31:0] haddr;
wire hwrite;
wire [2:0] hsize;
wire [2:0] hburst;
wire [31:0] hwdata;
wire hsel;
wire [1:0] htrans;

wire resetn;
wire resetn_auto;
reg [15:0] rstdly = 0;


wire mrstn;

//cnt instantiation
cnt u_cnt
(
  .clk(clk_in),
  .in(resetn_in),
  .out(mrstn)
);


always @(negedge clk_in)
begin
    if(!mrstn)
        rstdly <= 0;
    else
        rstdly <= {rstdly[14:0],1'b1};
end

assign resetn_auto = rstdly[15];
assign resetn = mrstn & resetn_auto;

//irq testing
wire irq_out[1:0];

//button instantiation
button irq_20_button
(
  .clk(clk_in), 
  .rstn(resetn), 
  .in(irq_in[0]), 
  .out(irq_out[0])
);

//button instantiation
button irq_21_button
(
  .clk(clk_in), 
  .rstn(resetn), 
  .in(irq_in[1]), 
  .out(irq_out[1])
);


//Gowin_PicoRV32_Top instantiation
Gowin_PicoRV32_Top u_Gowin_PicoRV32_Top
(
  .wbuart_tx(wbuart_tx),
  .wbuart_rx(wbuart_rx),
  .gpio_io(gpio_io),
  .slv_ext_stb_o(slv_ext_stb_o),
  .slv_ext_we_o(slv_ext_we_o),
  .slv_ext_cyc_o(slv_ext_cyc_o),
  .slv_ext_ack_i(slv_ext_ack_i),
  .slv_ext_adr_o(slv_ext_adr_o),
  .slv_ext_wdata_o(slv_ext_wdata_o),
  .slv_ext_rdata_i(slv_ext_rdata_i),
  .slv_ext_sel_o(slv_ext_sel_o),
  .io_spi_clk(io_spi_clk),
  .io_spi_csn(io_spi_csn),
  .io_spi_mosi(io_spi_mosi),
  .io_spi_miso(io_spi_miso),
  .hrdata(hrdata),
  .hresp(hresp),
  .hready(hready),
  .haddr(haddr),
  .hwrite(hwrite),
  .hsize(hsize),
  .hburst(hburst),
  .hwdata(hwdata),
  .hsel(hsel),
  .htrans(htrans),
  .irq_in({10'h0,irq_out[1],irq_out[0]}),
  .jtag_TDI(jtag_TDI),
  .jtag_TDO(jtag_TDO),
  .jtag_TCK(jtag_TCK),
  .jtag_TMS(jtag_TMS),
  .clk_in(clk_in),
  .resetn_in(resetn)
);


//wbreg_demo instantiation
wbreg_demo wbreg_demo_inst
(
  .wb_clk_i (clk_in), 
  .wb_rst_i (~resetn), 
  .arst_i   (~resetn), 
  .wb_adr_i (slv_ext_adr_o[4:0]), 
  .wb_dat_i (slv_ext_wdata_o), 
  .wb_dat_o (slv_ext_rdata_i),
  .wb_we_i  (slv_ext_we_o), 
  .wb_stb_i (slv_ext_stb_o), 
  .wb_cyc_i (slv_ext_cyc_o), 
  .wb_ack_o (slv_ext_ack_i)
);


//ahbreg_demo instantiation
ahbreg_demo ahbreg_demo_inst
(
  .hclk      (clk_in),
  .hresetn   (resetn),
  .haddr     (haddr),
  .htrans    (htrans),
  .hwrite    (hwrite),
  .hsize     (hsize),
  .hburst    (hburst),
  .hwdata    (hwdata),
  .hsel      (hsel),
  .hready_in (hready),
  .hready    (hready),
  .hrdata    (hrdata),
  .hresp     (hresp)
);

endmodule